
/*  A Bison parser, made from /5g/ian/binutils/release/copy/binutils/rcparse.y
 by  GNU Bison version 1.25
  */

#define YYBISON 1  /* Identify Bison output.  */

#define	BEG	258
#define	END	259
#define	ACCELERATORS	260
#define	VIRTKEY	261
#define	ASCII	262
#define	NOINVERT	263
#define	SHIFT	264
#define	CONTROL	265
#define	ALT	266
#define	BITMAP	267
#define	CURSOR	268
#define	DIALOG	269
#define	DIALOGEX	270
#define	EXSTYLE	271
#define	CAPTION	272
#define	CLASS	273
#define	STYLE	274
#define	AUTO3STATE	275
#define	AUTOCHECKBOX	276
#define	AUTORADIOBUTTON	277
#define	CHECKBOX	278
#define	COMBOBOX	279
#define	CTEXT	280
#define	DEFPUSHBUTTON	281
#define	EDITTEXT	282
#define	GROUPBOX	283
#define	LISTBOX	284
#define	LTEXT	285
#define	PUSHBOX	286
#define	PUSHBUTTON	287
#define	RADIOBUTTON	288
#define	RTEXT	289
#define	SCROLLBAR	290
#define	STATE3	291
#define	USERBUTTON	292
#define	BEDIT	293
#define	HEDIT	294
#define	IEDIT	295
#define	FONT	296
#define	ICON	297
#define	LANGUAGE	298
#define	CHARACTERISTICS	299
#define	VERSIONK	300
#define	MENU	301
#define	MENUEX	302
#define	MENUITEM	303
#define	SEPARATOR	304
#define	POPUP	305
#define	CHECKED	306
#define	GRAYED	307
#define	HELP	308
#define	INACTIVE	309
#define	MENUBARBREAK	310
#define	MENUBREAK	311
#define	MESSAGETABLE	312
#define	RCDATA	313
#define	STRINGTABLE	314
#define	VERSIONINFO	315
#define	FILEVERSION	316
#define	PRODUCTVERSION	317
#define	FILEFLAGSMASK	318
#define	FILEFLAGS	319
#define	FILEOS	320
#define	FILETYPE	321
#define	FILESUBTYPE	322
#define	BLOCKSTRINGFILEINFO	323
#define	BLOCKVARFILEINFO	324
#define	VALUE	325
#define	BLOCK	326
#define	MOVEABLE	327
#define	FIXED	328
#define	PURE	329
#define	IMPURE	330
#define	PRELOAD	331
#define	LOADONCALL	332
#define	DISCARDABLE	333
#define	NOT	334
#define	QUOTEDSTRING	335
#define	STRING	336
#define	NUMBER	337
#define	SIZEDSTRING	338
#define	NEG	339

#line 1 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
 /* rcparse.y -- parser for Windows rc files
   Copyright 1997 Free Software Foundation, Inc.
   Written by Ian Lance Taylor, Cygnus Support.

   This file is part of GNU Binutils.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.  */

/* This is a parser for Windows rc files.  It is based on the parser
   by Gunther Ebert <gunther.ebert@ixos-leipzig.de>.  */

#include "bfd.h"
#include "bucomm.h"
#include "libiberty.h"
#include "windres.h"

#include <ctype.h>

/* The current language.  */

static unsigned short language;

/* The resource information during a sub statement.  */

static struct res_res_info sub_res_info;

/* Dialog information.  This is built by the nonterminals styles and
   controls.  */

static struct dialog dialog;

/* This is used when building a style.  It is modified by the
   nonterminal styleexpr.  */

static unsigned long style;

/* These are used when building a control.  They are set before using
   control_params.  */

static unsigned long base_style;
static unsigned long default_style;
static unsigned long class;


#line 59 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
typedef union
{
  struct accelerator acc;
  struct accelerator *pacc;
  struct dialog_control *dialog_control;
  struct menuitem *menuitem;
  struct
  {
    struct rcdata_item *first;
    struct rcdata_item *last;
  } rcdata;
  struct rcdata_item *rcdata_item;
  struct stringtable_data *stringtable;
  struct fixed_versioninfo *fixver;
  struct ver_info *verinfo;
  struct ver_stringinfo *verstring;
  struct ver_varinfo *vervar;
  struct res_id id;
  struct res_res_info res_info;
  struct
  {
    unsigned short on;
    unsigned short off;
  } memflags;
  struct
  {
    unsigned long val;
    /* Nonzero if this number was explicitly specified as long.  */
    int dword;
  } i;
  unsigned long il;
  unsigned short is;
  const char *s;
  struct
  {
    unsigned long length;
    const char *s;
  } ss;
} YYSTYPE;
#include <stdio.h>

#ifndef __cplusplus
#ifndef __STDC__
#define const
#endif
#endif



#define	YYFINAL		462
#define	YYFLAG		-32768
#define	YYNTBASE	98

#define YYTRANSLATE(x) ((unsigned)(x) <= 339 ? yytranslate[x] : 188)

static const char yytranslate[] = {     0,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,    91,    86,     2,    96,
    97,    89,    87,    94,    88,     2,    90,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
    95,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,    85,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,    84,     2,    92,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     1,     2,     3,     4,     5,
     6,     7,     8,     9,    10,    11,    12,    13,    14,    15,
    16,    17,    18,    19,    20,    21,    22,    23,    24,    25,
    26,    27,    28,    29,    30,    31,    32,    33,    34,    35,
    36,    37,    38,    39,    40,    41,    42,    43,    44,    45,
    46,    47,    48,    49,    50,    51,    52,    53,    54,    55,
    56,    57,    58,    59,    60,    61,    62,    63,    64,    65,
    66,    67,    68,    69,    70,    71,    72,    73,    74,    75,
    76,    77,    78,    79,    80,    81,    82,    83,    93
};

#if YYDEBUG != 0
static const short yyprhs[] = {     0,
     0,     1,     5,     9,    13,    17,    21,    25,    29,    33,
    37,    41,    45,    49,    53,    57,    58,    65,    66,    69,
    72,    77,    79,    81,    83,    87,    90,    92,    94,    96,
    98,   100,   102,   107,   112,   113,   127,   128,   142,   143,
   158,   159,   163,   164,   168,   172,   173,   178,   182,   188,
   196,   200,   204,   209,   213,   214,   217,   218,   222,   223,
   227,   228,   232,   233,   237,   238,   242,   243,   247,   259,
   272,   273,   277,   278,   282,   283,   287,   288,   292,   293,
   297,   304,   315,   327,   328,   332,   333,   337,   338,   342,
   343,   347,   348,   352,   353,   357,   358,   362,   363,   367,
   368,   372,   373,   390,   398,   408,   419,   420,   423,   424,
   428,   429,   433,   434,   438,   439,   443,   448,   453,   457,
   464,   465,   468,   473,   476,   483,   484,   488,   491,   493,
   495,   497,   499,   501,   503,   510,   511,   514,   517,   521,
   527,   533,   540,   548,   558,   563,   570,   571,   574,   575,
   577,   579,   581,   585,   589,   590,   597,   598,   602,   607,
   614,   619,   626,   627,   634,   641,   645,   649,   653,   657,
   661,   662,   671,   679,   680,   686,   687,   691,   693,   695,
   696,   699,   703,   708,   712,   713,   716,   717,   720,   722,
   724,   726,   728,   730,   732,   734,   736,   738,   740,   743,
   747,   752,   754,   758,   759,   761,   764,   766,   768,   772,
   775,   778,   782,   786,   790,   794,   798,   802,   806,   810,
   813,   815,   817,   821,   824,   828,   832,   836,   840,   844,
   848,   852
};

static const short yyrhs[] = {    -1,
    98,    99,   100,     0,    98,    99,   106,     0,    98,    99,
   107,     0,    98,    99,   108,     0,    98,    99,   147,     0,
    98,    99,   148,     0,    98,    99,   149,     0,    98,    99,
   150,     0,    98,    99,   155,     0,    98,    99,   158,     0,
    98,    99,   159,     0,    98,    99,   164,     0,    98,    99,
   167,     0,    98,    99,   168,     0,     0,   173,     5,   174,
     3,   101,     4,     0,     0,   101,   102,     0,   103,   185,
     0,   103,   185,    94,   104,     0,    80,     0,   186,     0,
   105,     0,   104,    94,   105,     0,   104,   105,     0,     6,
     0,     7,     0,     8,     0,     9,     0,    10,     0,    11,
     0,   173,    12,   176,   178,     0,   173,    13,   175,   178,
     0,     0,   173,    14,   176,   112,   186,   182,   182,   182,
   109,   113,     3,   115,     4,     0,     0,   173,    15,   176,
   112,   186,   182,   182,   182,   110,   113,     3,   115,     4,
     0,     0,   173,    15,   176,   112,   186,   182,   182,   182,
   182,   111,   113,     3,   115,     4,     0,     0,    16,    95,
   183,     0,     0,   113,    17,    80,     0,   113,    18,   173,
     0,     0,   113,    19,   114,   179,     0,   113,    16,   183,
     0,   113,    41,   183,    94,    80,     0,   113,    41,   183,
    94,    80,   182,   182,     0,   113,    46,   173,     0,   113,
    44,   183,     0,   113,    43,   183,   182,     0,   113,    45,
   183,     0,     0,   115,   116,     0,     0,    20,   117,   138,
     0,     0,    21,   118,   138,     0,     0,    22,   119,   138,
     0,     0,    38,   120,   138,     0,     0,    23,   121,   138,
     0,     0,    24,   122,   138,     0,    10,   139,   183,   182,
   141,   182,   182,   182,   182,   181,   140,     0,    10,   139,
   183,   182,   141,   182,   182,   182,   182,   182,   182,   140,
     0,     0,    25,   123,   138,     0,     0,    26,   124,   138,
     0,     0,    27,   125,   138,     0,     0,    28,   126,   138,
     0,     0,    39,   127,   138,     0,    42,   139,   183,   182,
   182,   140,     0,    42,   139,   183,   182,   182,   182,   182,
   143,   181,   140,     0,    42,   139,   183,   182,   182,   182,
   182,   143,   182,   182,   140,     0,     0,    40,   128,   138,
     0,     0,    29,   129,   138,     0,     0,    30,   130,   138,
     0,     0,    31,   131,   138,     0,     0,    32,   132,   138,
     0,     0,    33,   133,   138,     0,     0,    34,   134,   138,
     0,     0,    35,   135,   138,     0,     0,    36,   136,   138,
     0,     0,    37,    80,    94,   183,    94,   183,    94,   183,
    94,   183,    94,   183,    94,   137,   179,   181,     0,   139,
   183,   182,   182,   182,   182,   140,     0,   139,   183,   182,
   182,   182,   182,   145,   181,   140,     0,   139,   183,   182,
   182,   182,   182,   145,   182,   182,   140,     0,     0,    80,
    94,     0,     0,     3,   160,     4,     0,     0,    94,   142,
   179,     0,     0,    94,   144,   179,     0,     0,    94,   146,
   179,     0,   173,    41,   175,   178,     0,   173,    42,   175,
   178,     0,    43,   183,   182,     0,   173,    46,   174,     3,
   151,     4,     0,     0,   151,   152,     0,    48,    80,   182,
   153,     0,    48,    49,     0,    50,    80,   153,     3,   151,
     4,     0,     0,   153,    94,   154,     0,   153,   154,     0,
    51,     0,    52,     0,    53,     0,    54,     0,    55,     0,
    56,     0,   173,    47,   174,     3,   156,     4,     0,     0,
   156,   157,     0,    48,    80,     0,    48,    80,   182,     0,
    48,    80,   182,   182,   181,     0,    50,    80,     3,   156,
     4,     0,    50,    80,   182,     3,   156,     4,     0,    50,
    80,   182,   182,     3,   156,     4,     0,    50,    80,   182,
   182,   182,   181,     3,   156,     4,     0,   173,    57,   176,
   178,     0,   173,    58,   174,     3,   160,     4,     0,     0,
   161,   162,     0,     0,   163,     0,    83,     0,   184,     0,
   163,    94,    83,     0,   163,    94,   184,     0,     0,    59,
   174,     3,   165,   166,     4,     0,     0,   166,   183,    80,
     0,   166,   183,    94,    80,     0,   173,   173,   174,     3,
   160,     4,     0,   173,   173,   174,   178,     0,   173,    60,
   169,     3,   170,     4,     0,     0,   169,    61,   183,   182,
   182,   182,     0,   169,    62,   183,   182,   182,   182,     0,
   169,    63,   183,     0,   169,    64,   183,     0,   169,    65,
   183,     0,   169,    66,   183,     0,   169,    67,   183,     0,
     0,   170,    68,     3,    71,     3,   171,     4,     4,     0,
   170,    69,     3,    70,    80,   172,     4,     0,     0,   171,
    70,    80,    94,    80,     0,     0,   172,   182,   182,     0,
   186,     0,    81,     0,     0,   174,   177,     0,   174,    44,
   183,     0,   174,    43,   183,   182,     0,   174,    45,   183,
     0,     0,   175,   177,     0,     0,   175,   177,     0,    72,
     0,    73,     0,    74,     0,    75,     0,    76,     0,    77,
     0,    78,     0,    80,     0,    81,     0,   180,     0,    79,
   180,     0,   179,    84,   180,     0,   179,    84,    79,   180,
     0,    82,     0,    96,   183,    97,     0,     0,   182,     0,
    94,   183,     0,   184,     0,    82,     0,    96,   184,    97,
     0,    92,   184,     0,    88,   184,     0,   184,    89,   184,
     0,   184,    90,   184,     0,   184,    91,   184,     0,   184,
    87,   184,     0,   184,    88,   184,     0,   184,    86,   184,
     0,   184,    85,   184,     0,   184,    84,   184,     0,    94,
   186,     0,   187,     0,    82,     0,    96,   184,    97,     0,
    92,   184,     0,   187,    89,   184,     0,   187,    90,   184,
     0,   187,    91,   184,     0,   187,    87,   184,     0,   187,
    88,   184,     0,   187,    86,   184,     0,   187,    85,   184,
     0,   187,    84,   184,     0
};

#endif

#if YYDEBUG != 0
static const short yyrline[] = { 0,
   154,   156,   157,   158,   159,   160,   161,   162,   163,   164,
   165,   166,   167,   168,   169,   172,   181,   188,   193,   213,
   219,   230,   252,   261,   266,   271,   277,   282,   287,   291,
   295,   299,   307,   316,   325,   343,   347,   366,   370,   390,
   396,   401,   407,   409,   413,   417,   420,   423,   427,   433,
   446,   450,   454,   458,   464,   466,   476,   484,   487,   494,
   497,   504,   507,   514,   520,   527,   530,   537,   540,   551,
   560,   567,   570,   577,   580,   587,   590,   597,   600,   607,
   613,   624,   636,   646,   653,   659,   666,   669,   676,   679,
   686,   689,   696,   699,   706,   709,   716,   719,   726,   729,
   736,   739,   742,   756,   769,   780,   791,   796,   802,   807,
   815,   819,   821,   825,   827,   831,   835,   844,   854,   863,
   870,   875,   891,   896,   900,   906,   911,   915,   921,   926,
   930,   934,   938,   942,   950,   957,   962,   978,   983,   987,
   991,   995,   999,  1003,  1012,  1021,  1031,  1036,  1042,  1048,
  1054,  1063,  1071,  1080,  1093,  1096,  1099,  1101,  1105,  1114,
  1119,  1127,  1134,  1141,  1147,  1153,  1158,  1163,  1168,  1173,
  1186,  1191,  1195,  1201,  1206,  1212,  1217,  1225,  1231,  1248,
  1256,  1262,  1267,  1272,  1281,  1288,  1298,  1305,  1316,  1322,
  1327,  1332,  1337,  1342,  1347,  1356,  1361,  1377,  1382,  1386,
  1390,  1396,  1401,  1409,  1414,  1422,  1431,  1440,  1445,  1449,
  1454,  1459,  1464,  1469,  1474,  1479,  1484,  1489,  1494,  1504,
  1513,  1524,  1529,  1533,  1538,  1543,  1548,  1553,  1558,  1563,
  1568,  1573
};
#endif


#if YYDEBUG != 0 || defined (YYERROR_VERBOSE)

static const char * const yytname[] = {   "$","error","$undefined.","BEG","END",
"ACCELERATORS","VIRTKEY","ASCII","NOINVERT","SHIFT","CONTROL","ALT","BITMAP",
"CURSOR","DIALOG","DIALOGEX","EXSTYLE","CAPTION","CLASS","STYLE","AUTO3STATE",
"AUTOCHECKBOX","AUTORADIOBUTTON","CHECKBOX","COMBOBOX","CTEXT","DEFPUSHBUTTON",
"EDITTEXT","GROUPBOX","LISTBOX","LTEXT","PUSHBOX","PUSHBUTTON","RADIOBUTTON",
"RTEXT","SCROLLBAR","STATE3","USERBUTTON","BEDIT","HEDIT","IEDIT","FONT","ICON",
"LANGUAGE","CHARACTERISTICS","VERSIONK","MENU","MENUEX","MENUITEM","SEPARATOR",
"POPUP","CHECKED","GRAYED","HELP","INACTIVE","MENUBARBREAK","MENUBREAK","MESSAGETABLE",
"RCDATA","STRINGTABLE","VERSIONINFO","FILEVERSION","PRODUCTVERSION","FILEFLAGSMASK",
"FILEFLAGS","FILEOS","FILETYPE","FILESUBTYPE","BLOCKSTRINGFILEINFO","BLOCKVARFILEINFO",
"VALUE","BLOCK","MOVEABLE","FIXED","PURE","IMPURE","PRELOAD","LOADONCALL","DISCARDABLE",
"NOT","QUOTEDSTRING","STRING","NUMBER","SIZEDSTRING","'|'","'^'","'&'","'+'",
"'-'","'*'","'/'","'%'","'~'","NEG","','","'='","'('","')'","input","newcmd",
"accelerator","acc_entries","acc_entry","acc_event","acc_options","acc_option",
"bitmap","cursor","dialog","@1","@2","@3","exstyle","styles","@4","controls",
"control","@5","@6","@7","@8","@9","@10","@11","@12","@13","@14","@15","@16",
"@17","@18","@19","@20","@21","@22","@23","@24","@25","control_params","optstringc",
"opt_control_data","control_styleexpr","@26","icon_styleexpr","@27","control_params_styleexpr",
"@28","font","icon","language","menu","menuitems","menuitem","menuitem_flags",
"menuitem_flag","menuex","menuexitems","menuexitem","messagetable","rcdata",
"optrcdata_data","@29","optrcdata_data_int","rcdata_data","stringtable","@30",
"string_data","user","versioninfo","fixedverinfo","verblocks","vervals","vertrans",
"id","suboptions","memflags_move_discard","memflags_move","memflag","file_name",
"styleexpr","parennumber","optcnumexpr","cnumexpr","numexpr","sizednumexpr",
"cposnumexpr","posnumexpr","sizedposnumexpr", NULL
};
#endif

static const short yyr1[] = {     0,
    98,    98,    98,    98,    98,    98,    98,    98,    98,    98,
    98,    98,    98,    98,    98,    99,   100,   101,   101,   102,
   102,   103,   103,   104,   104,   104,   105,   105,   105,   105,
   105,   105,   106,   107,   109,   108,   110,   108,   111,   108,
   112,   112,   113,   113,   113,   114,   113,   113,   113,   113,
   113,   113,   113,   113,   115,   115,   117,   116,   118,   116,
   119,   116,   120,   116,   121,   116,   122,   116,   116,   116,
   123,   116,   124,   116,   125,   116,   126,   116,   127,   116,
   116,   116,   116,   128,   116,   129,   116,   130,   116,   131,
   116,   132,   116,   133,   116,   134,   116,   135,   116,   136,
   116,   137,   116,   138,   138,   138,   139,   139,   140,   140,
   142,   141,   144,   143,   146,   145,   147,   148,   149,   150,
   151,   151,   152,   152,   152,   153,   153,   153,   154,   154,
   154,   154,   154,   154,   155,   156,   156,   157,   157,   157,
   157,   157,   157,   157,   158,   159,   161,   160,   162,   162,
   163,   163,   163,   163,   165,   164,   166,   166,   166,   167,
   167,   168,   169,   169,   169,   169,   169,   169,   169,   169,
   170,   170,   170,   171,   171,   172,   172,   173,   173,   174,
   174,   174,   174,   174,   175,   175,   176,   176,   177,   177,
   177,   177,   177,   177,   177,   178,   178,   179,   179,   179,
   179,   180,   180,   181,   181,   182,   183,   184,   184,   184,
   184,   184,   184,   184,   184,   184,   184,   184,   184,   185,
   186,   187,   187,   187,   187,   187,   187,   187,   187,   187,
   187,   187
};

static const short yyr2[] = {     0,
     0,     3,     3,     3,     3,     3,     3,     3,     3,     3,
     3,     3,     3,     3,     3,     0,     6,     0,     2,     2,
     4,     1,     1,     1,     3,     2,     1,     1,     1,     1,
     1,     1,     4,     4,     0,    13,     0,    13,     0,    14,
     0,     3,     0,     3,     3,     0,     4,     3,     5,     7,
     3,     3,     4,     3,     0,     2,     0,     3,     0,     3,
     0,     3,     0,     3,     0,     3,     0,     3,    11,    12,
     0,     3,     0,     3,     0,     3,     0,     3,     0,     3,
     6,    10,    11,     0,     3,     0,     3,     0,     3,     0,
     3,     0,     3,     0,     3,     0,     3,     0,     3,     0,
     3,     0,    16,     7,     9,    10,     0,     2,     0,     3,
     0,     3,     0,     3,     0,     3,     4,     4,     3,     6,
     0,     2,     4,     2,     6,     0,     3,     2,     1,     1,
     1,     1,     1,     1,     6,     0,     2,     2,     3,     5,
     5,     6,     7,     9,     4,     6,     0,     2,     0,     1,
     1,     1,     3,     3,     0,     6,     0,     3,     4,     6,
     4,     6,     0,     6,     6,     3,     3,     3,     3,     3,
     0,     8,     7,     0,     5,     0,     3,     1,     1,     0,
     2,     3,     4,     3,     0,     2,     0,     2,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     2,     3,
     4,     1,     3,     0,     1,     2,     1,     1,     3,     2,
     2,     3,     3,     3,     3,     3,     3,     3,     3,     2,
     1,     1,     3,     2,     3,     3,     3,     3,     3,     3,
     3,     3
};

static const short yydefact[] = {     1,
    16,     0,     0,   180,   179,   222,     0,     0,     2,     3,
     4,     5,     6,     7,     8,     9,    10,    11,    12,    13,
    14,    15,     0,   178,   221,   208,     0,     0,     0,     0,
   207,     0,   224,     0,   180,   185,   185,   185,   185,   185,
   185,   180,   180,   185,   180,   163,   180,     0,     0,     0,
     0,     0,     0,     0,     0,   211,   210,     0,     0,   119,
     0,     0,     0,     0,     0,     0,     0,     0,   155,     0,
     0,     0,   189,   190,   191,   192,   193,   194,   195,   181,
   223,     0,     0,     0,     0,    41,    41,     0,     0,     0,
     0,     0,     0,     0,     0,   232,   231,   230,   228,   229,
   225,   226,   227,   209,   206,   219,   218,   217,   215,   216,
   212,   213,   214,   157,     0,   182,   184,    18,   186,   196,
   197,    33,   186,    34,     0,     0,     0,   117,   118,   121,
   136,   145,   147,   171,     0,     0,     0,     0,     0,     0,
     0,   147,   161,     0,   183,     0,     0,     0,     0,     0,
     0,     0,   149,     0,     0,     0,   166,   167,   168,   169,
   170,     0,   156,     0,    17,    22,    19,     0,    23,    42,
     0,     0,   120,     0,     0,   122,   135,     0,     0,   137,
   146,   151,   148,   150,   152,   162,     0,     0,     0,     0,
   160,   158,     0,     0,    20,     0,     0,   124,     0,   126,
   138,     0,     0,     0,     0,     0,     0,   159,   220,     0,
    35,    37,   126,     0,   139,   136,     0,   153,   154,     0,
     0,   164,   165,    27,    28,    29,    30,    31,    32,    21,
    24,    43,    43,    39,   123,   121,   129,   130,   131,   132,
   133,   134,     0,   128,   204,     0,   136,     0,   174,   176,
     0,    26,     0,     0,    43,     0,   127,   140,   205,   141,
     0,   136,   204,     0,     0,    25,    55,     0,     0,     0,
    46,     0,     0,     0,     0,     0,    55,     0,   125,   142,
     0,     0,     0,     0,   173,     0,     0,    48,    44,    45,
     0,     0,     0,    52,    54,    51,     0,    55,   143,   136,
   172,     0,   177,    36,   107,    57,    59,    61,    65,    67,
    71,    73,    75,    77,    86,    88,    90,    92,    94,    96,
    98,   100,     0,    63,    79,    84,   107,    56,     0,   202,
     0,    47,   198,     0,    53,    38,     0,     0,     0,     0,
     0,   107,   107,   107,   107,   107,   107,   107,   107,   107,
   107,   107,   107,   107,   107,   107,   107,   107,     0,   107,
   107,   107,     0,   199,     0,     0,    49,    40,   144,   175,
   108,     0,    58,     0,    60,    62,    66,    68,    72,    74,
    76,    78,    87,    89,    91,    93,    95,    97,    99,   101,
     0,    64,    80,    85,     0,   203,     0,   200,     0,     0,
     0,     0,     0,   201,    50,   111,     0,     0,     0,   109,
     0,     0,     0,     0,   147,    81,     0,   112,     0,     0,
     0,     0,     0,     0,   109,     0,   110,   113,   204,   204,
   115,   104,   204,     0,     0,   109,   205,   109,   205,     0,
   109,   205,     0,   114,    82,   109,    69,   109,   116,   105,
   109,     0,    83,    70,   106,     0,   102,     0,   204,   103,
     0,     0
};

static const short yydefgoto[] = {     1,
     2,     9,   146,   167,   168,   230,   231,    10,    11,    12,
   232,   233,   255,   126,   253,   291,   287,   328,   342,   343,
   344,   360,   345,   346,   347,   348,   349,   350,   361,   362,
   351,   352,   353,   354,   355,   356,   357,   358,   458,   373,
   374,   416,   407,   411,   429,   435,   433,   440,    13,    14,
    15,    16,   150,   176,   214,   244,    17,   151,   180,    18,
    19,   152,   153,   183,   184,    20,   114,   144,    21,    22,
    94,   154,   264,   265,    23,    32,    83,    84,    80,   122,
   332,   333,   258,   259,    30,    31,   195,    24,    25
};

static const short yypact[] = {-32768,
    41,    97,   496,-32768,-32768,-32768,   496,   496,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,   149,-32768,   419,-32768,   496,   496,   496,   -49,
   456,   214,-32768,   533,-32768,   -75,-32768,   -12,   -12,-32768,
-32768,-32768,-32768,   -75,-32768,-32768,-32768,   496,   496,   496,
   496,   496,   496,   496,   496,-32768,-32768,   547,   496,-32768,
   496,   496,   496,   496,   496,   496,   496,   496,-32768,   496,
   496,   496,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,   234,   583,     2,   573,    95,    95,   573,   573,   257,
   277,     2,   341,   295,   195,   577,   366,   538,   163,   163,
-32768,-32768,-32768,-32768,-32768,   577,   366,   538,   163,   163,
-32768,-32768,-32768,-32768,   -49,-32768,-32768,-32768,    53,-32768,
-32768,-32768,-32768,-32768,    33,   123,   123,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,   496,   496,   496,   496,   496,   496,
   496,-32768,-32768,    14,-32768,    17,   496,   -49,   -49,    38,
   100,   127,   231,    55,   -49,   -49,-32768,-32768,-32768,-32768,
-32768,   132,-32768,   -77,-32768,-32768,-32768,    45,-32768,-32768,
   -49,   -49,-32768,    66,    63,-32768,-32768,    64,    67,-32768,
-32768,-32768,-32768,    76,   456,-32768,   171,   180,   -49,   -49,
-32768,-32768,   106,   123,    98,   -49,   -49,-32768,   -49,-32768,
   -49,     6,   309,   128,   131,   -49,   -49,-32768,-32768,   663,
-32768,   -49,-32768,    20,   -49,-32768,     9,-32768,   456,   205,
   136,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,    26,
-32768,-32768,-32768,-32768,   376,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,   624,-32768,   -49,   125,-32768,    13,-32768,-32768,
   663,-32768,   405,   417,-32768,   154,-32768,-32768,-32768,-32768,
   164,-32768,   -49,     7,    11,-32768,-32768,   496,   144,   169,
-32768,   496,   496,   496,   496,   169,-32768,   423,-32768,-32768,
   184,   230,   232,   162,-32768,   -49,   455,-32768,-32768,-32768,
   -39,   153,   -49,-32768,-32768,-32768,   492,-32768,-32768,-32768,
-32768,   168,-32768,-32768,   186,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,   201,-32768,-32768,-32768,   186,-32768,    34,-32768,
   496,   159,-32768,   202,-32768,-32768,   529,   196,   203,   170,
   496,   186,   186,   186,   186,   186,   186,   186,   186,   186,
   186,   186,   186,   186,   186,   186,   186,   186,   190,   186,
   186,   186,   496,-32768,   151,   115,   -49,-32768,-32768,-32768,
-32768,   -49,-32768,   496,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
   496,-32768,-32768,-32768,   -49,-32768,    34,-32768,   -49,   200,
   -49,   209,   -49,-32768,-32768,-32768,   -49,   -49,   496,    27,
   -39,   -49,   -49,   210,-32768,-32768,   -49,   159,   -49,   -49,
   496,   301,   221,   -49,    28,   222,-32768,-32768,   -49,   -49,
-32768,-32768,   -49,   496,   -39,   296,   -49,   296,   -49,   -39,
   296,   -49,   224,   159,-32768,   296,-32768,   296,   159,-32768,
   296,   496,-32768,-32768,-32768,   244,-32768,   -39,   -38,-32768,
   324,-32768
};

static const short yypgoto[] = {-32768,
-32768,-32768,-32768,-32768,-32768,-32768,  -172,-32768,-32768,-32768,
-32768,-32768,-32768,   238,  -211,-32768,  -160,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,   254,
  -267,   145,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,    90,-32768,   134,   105,-32768,  -135,-32768,-32768,
-32768,  -141,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,   -21,   178,   256,   113,   491,   251,
  -303,  -319,  -249,   -30,   -46,     0,-32768,  -107,-32768
};


#define	YYLAST		680


static const short yytable[] = {    60,
   162,    47,   192,  -187,  -187,  -187,    33,    34,   216,   364,
   283,   247,   105,   282,   285,   262,   193,   163,   148,   149,
   165,   254,   236,   115,   116,   117,    56,    57,    58,   415,
   415,   224,   225,   226,   227,   228,   229,   341,   169,   329,
   461,   173,   330,   278,    59,   366,   398,    96,    97,    98,
    99,   100,   101,   102,   103,    59,   331,   252,   186,   363,
   106,   107,   108,   109,   110,   111,   112,   113,  -188,  -187,
   237,   238,   239,   240,   241,   242,   284,   404,   266,  -187,
   246,   120,   121,  -187,   145,   174,   209,   175,   155,   156,
   157,   158,   159,   160,   161,    26,   166,   164,     6,    59,
   170,    27,    59,   177,    59,    28,    59,   418,     7,    29,
   125,   261,     8,   243,   198,   330,   297,   171,   172,   251,
    59,   431,   187,   188,   189,   190,   281,   147,   260,   331,
   181,   444,  -188,  -188,  -188,   191,   449,   337,   194,     3,
   196,   197,   200,   201,  -188,   199,   202,   178,  -188,   179,
    86,    87,   185,    35,   459,     4,    92,   279,   206,   207,
    36,    37,    38,    39,   338,   211,   212,   280,   213,   203,
   215,   217,   178,   204,   179,   222,   223,     5,     6,   436,
   438,   234,   205,   441,   245,   208,   248,   299,     7,    40,
    41,   210,     8,   397,    42,    43,   330,   142,   220,   369,
   221,   174,   219,   175,     6,    44,    45,   249,    46,   460,
   331,   178,    82,   179,     7,   250,    69,   263,     8,    90,
    91,   288,    93,   289,    95,   292,   293,   294,   295,     5,
     6,   178,   300,   179,   286,   301,   118,    70,    71,    72,
     7,   302,   366,   178,     8,   179,   334,   396,   290,     5,
     6,    66,    67,    68,   296,   303,    70,    71,    72,   130,
     7,   339,   335,   371,     8,   340,    73,    74,    75,    76,
    77,    78,    79,   422,   120,   121,    70,    71,    72,   131,
   359,   367,   370,   391,   365,    73,    74,    75,    76,    77,
    78,    79,    85,   406,   372,    88,    89,   134,   415,    70,
    71,    72,   409,   421,   427,    73,    74,    75,    76,    77,
    78,    79,    26,   182,   428,   434,   395,   452,    27,    70,
    71,    72,    28,   462,   127,   256,    29,   401,    73,    74,
    75,    76,    77,    78,    79,   124,   399,   457,   128,   129,
     0,   400,   132,   133,   402,   143,   235,   257,    73,    74,
    75,    76,    77,    78,    79,   135,   136,   137,   138,   139,
   140,   141,   414,     0,   403,     0,     0,     0,   405,     0,
   408,     0,   410,     0,   426,     0,   412,   413,     0,   417,
     0,   419,   420,    70,    71,    72,   423,   443,   424,   425,
    26,   218,     0,   430,     0,     0,    27,     0,   437,   439,
    28,     0,   442,     0,    29,   456,   446,   267,   448,     0,
     0,   451,    73,    74,    75,    76,    77,    78,    79,   277,
   268,   269,   270,   271,     0,   298,   237,   238,   239,   240,
   241,   242,   268,   269,   270,   271,     0,     0,   268,   269,
   270,   271,     0,     0,     0,   272,     0,   273,   274,   275,
   276,    63,    64,    65,    66,    67,    68,   272,   304,   273,
   274,   275,   276,   272,   305,   273,   274,   275,   276,   243,
     0,     0,     0,     0,   306,   307,   308,   309,   310,   311,
   312,   313,   314,   315,   316,   317,   318,   319,   320,   321,
   322,   323,   324,   325,   326,   336,   327,     0,     0,     0,
     0,   305,    48,    49,    50,    51,    52,    53,    54,    55,
     0,   306,   307,   308,   309,   310,   311,   312,   313,   314,
   315,   316,   317,   318,   319,   320,   321,   322,   323,   324,
   325,   326,   368,   327,     0,     0,     0,     0,   305,    61,
    62,    63,    64,    65,    66,    67,    68,     0,   306,   307,
   308,   309,   310,   311,   312,   313,   314,   315,   316,   317,
   318,   319,   320,   321,   322,   323,   324,   325,   326,   432,
   327,     0,     0,   119,     0,   123,     0,    26,   123,   123,
   445,     0,   447,    27,     0,   450,     0,    28,     0,     0,
   453,    29,   454,     0,     0,   455,   375,   376,   377,   378,
   379,   380,   381,   382,   383,   384,   385,   386,   387,   388,
   389,   390,     0,   392,   393,   394,    61,    62,    63,    64,
    65,    66,    67,    68,    64,    65,    66,    67,    68,    81,
    61,    62,    63,    64,    65,    66,    67,    68,     0,     0,
     0,     0,     0,   104,    73,    74,    75,    76,    77,    78,
    79,     0,   120,   121,    73,    74,    75,    76,    77,    78,
    79,    62,    63,    64,    65,    66,    67,    68,   224,   225,
   226,   227,   228,   229,   237,   238,   239,   240,   241,   242
};

static const short yycheck[] = {    30,
   142,    23,    80,    16,    80,    81,     7,     8,     3,   329,
     4,     3,    59,   263,     4,     3,    94,     4,   126,   127,
     4,   233,     3,    70,    71,    72,    27,    28,    29,     3,
     3,     6,     7,     8,     9,    10,    11,   305,   146,    79,
     0,     4,    82,   255,    94,    84,   366,    48,    49,    50,
    51,    52,    53,    54,    55,    94,    96,   230,     4,   327,
    61,    62,    63,    64,    65,    66,    67,    68,    16,    82,
    51,    52,    53,    54,    55,    56,    70,   397,   251,    92,
   216,    80,    81,    96,   115,    48,   194,    50,   135,   136,
   137,   138,   139,   140,   141,    82,    80,   144,    82,    94,
   147,    88,    94,     4,    94,    92,    94,   411,    92,    96,
    16,   247,    96,    94,    49,    82,   277,   148,   149,    94,
    94,    94,    68,    69,   155,   156,   262,    95,     4,    96,
     4,   435,    80,    81,    82,     4,   440,   298,    94,    43,
   171,   172,    80,    80,    92,    80,    80,    48,    96,    50,
    38,    39,   153,     5,   458,    59,    44,     4,   189,   190,
    12,    13,    14,    15,   300,   196,   197,     4,   199,    94,
   201,   202,    48,     3,    50,   206,   207,    81,    82,   429,
   430,   212,     3,   433,   215,    80,   217,     4,    92,    41,
    42,    94,    96,    79,    46,    47,    82,     3,    71,     4,
    70,    48,   203,    50,    82,    57,    58,     3,    60,   459,
    96,    48,    35,    50,    92,    80,     3,   248,    96,    42,
    43,   268,    45,    80,    47,   272,   273,   274,   275,    81,
    82,    48,     3,    50,   265,     4,     3,    43,    44,    45,
    92,    80,    84,    48,    96,    50,    94,    97,   270,    81,
    82,    89,    90,    91,   276,   286,    43,    44,    45,     3,
    92,    94,   293,    94,    96,    80,    72,    73,    74,    75,
    76,    77,    78,   415,    80,    81,    43,    44,    45,     3,
    80,    80,    80,    94,   331,    72,    73,    74,    75,    76,
    77,    78,    37,    94,   341,    40,    41,     3,     3,    43,
    44,    45,    94,    94,     4,    72,    73,    74,    75,    76,
    77,    78,    82,    83,    94,    94,   363,    94,    88,    43,
    44,    45,    92,     0,    87,   236,    96,   374,    72,    73,
    74,    75,    76,    77,    78,    85,   367,    94,    88,    89,
    -1,   372,    92,     3,   391,    95,   213,   243,    72,    73,
    74,    75,    76,    77,    78,    61,    62,    63,    64,    65,
    66,    67,   409,    -1,   395,    -1,    -1,    -1,   399,    -1,
   401,    -1,   403,    -1,   421,    -1,   407,   408,    -1,   410,
    -1,   412,   413,    43,    44,    45,   417,   434,   419,   420,
    82,    83,    -1,   424,    -1,    -1,    88,    -1,   429,   430,
    92,    -1,   433,    -1,    96,   452,   437,     3,   439,    -1,
    -1,   442,    72,    73,    74,    75,    76,    77,    78,     3,
    16,    17,    18,    19,    -1,     3,    51,    52,    53,    54,
    55,    56,    16,    17,    18,    19,    -1,    -1,    16,    17,
    18,    19,    -1,    -1,    -1,    41,    -1,    43,    44,    45,
    46,    86,    87,    88,    89,    90,    91,    41,     4,    43,
    44,    45,    46,    41,    10,    43,    44,    45,    46,    94,
    -1,    -1,    -1,    -1,    20,    21,    22,    23,    24,    25,
    26,    27,    28,    29,    30,    31,    32,    33,    34,    35,
    36,    37,    38,    39,    40,     4,    42,    -1,    -1,    -1,
    -1,    10,    84,    85,    86,    87,    88,    89,    90,    91,
    -1,    20,    21,    22,    23,    24,    25,    26,    27,    28,
    29,    30,    31,    32,    33,    34,    35,    36,    37,    38,
    39,    40,     4,    42,    -1,    -1,    -1,    -1,    10,    84,
    85,    86,    87,    88,    89,    90,    91,    -1,    20,    21,
    22,    23,    24,    25,    26,    27,    28,    29,    30,    31,
    32,    33,    34,    35,    36,    37,    38,    39,    40,   425,
    42,    -1,    -1,    83,    -1,    85,    -1,    82,    88,    89,
   436,    -1,   438,    88,    -1,   441,    -1,    92,    -1,    -1,
   446,    96,   448,    -1,    -1,   451,   343,   344,   345,   346,
   347,   348,   349,   350,   351,   352,   353,   354,   355,   356,
   357,   358,    -1,   360,   361,   362,    84,    85,    86,    87,
    88,    89,    90,    91,    87,    88,    89,    90,    91,    97,
    84,    85,    86,    87,    88,    89,    90,    91,    -1,    -1,
    -1,    -1,    -1,    97,    72,    73,    74,    75,    76,    77,
    78,    -1,    80,    81,    72,    73,    74,    75,    76,    77,
    78,    85,    86,    87,    88,    89,    90,    91,     6,     7,
     8,     9,    10,    11,    51,    52,    53,    54,    55,    56
};
/* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
#line 3 "/usr/cygnus/progressive-97r2/share/bison.simple"

/* Skeleton output parser for bison,
   Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* As a special exception, when this file is copied by Bison into a
   Bison output file, you may use that output file without restriction.
   This special exception was added by the Free Software Foundation
   in version 1.24 of Bison.  */

#ifndef alloca
#ifdef __GNUC__
#define alloca __builtin_alloca
#else /* not GNU C.  */
#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__) || defined (__sparc) || defined (__sgi)
#include <alloca.h>
#else /* not sparc */
#if defined (MSDOS) && !defined (__TURBOC__)
#include <malloc.h>
#else /* not MSDOS, or __TURBOC__ */
#if defined(_AIX)
#include <malloc.h>
 #pragma alloca
#else /* not MSDOS, __TURBOC__, or _AIX */
#ifdef __hpux
#ifdef __cplusplus
extern "C" {
void *alloca (unsigned int);
};
#else /* not __cplusplus */
void *alloca ();
#endif /* not __cplusplus */
#endif /* __hpux */
#endif /* not _AIX */
#endif /* not MSDOS, or __TURBOC__ */
#endif /* not sparc.  */
#endif /* not GNU C.  */
#endif /* alloca not defined.  */

/* This is the parser code that is written into each bison parser
  when the %semantic_parser declaration is not specified in the grammar.
  It was written by Richard Stallman by simplifying the hairy parser
  used when %semantic_parser is specified.  */

/* Note: there must be only one dollar sign in this file.
   It is replaced by the list of actions, each action
   as one case of the switch.  */

#define yyerrok		(yyerrstatus = 0)
#define yyclearin	(yychar = YYEMPTY)
#define YYEMPTY		-2
#define YYEOF		0
#define YYACCEPT	return(0)
#define YYABORT 	return(1)
#define YYERROR		goto yyerrlab1
/* Like YYERROR except do call yyerror.
   This remains here temporarily to ease the
   transition to the new meaning of YYERROR, for GCC.
   Once GCC version 2 has supplanted version 1, this can go.  */
#define YYFAIL		goto yyerrlab
#define YYRECOVERING()  (!!yyerrstatus)
#define YYBACKUP(token, value) \
do								\
  if (yychar == YYEMPTY && yylen == 1)				\
    { yychar = (token), yylval = (value);			\
      yychar1 = YYTRANSLATE (yychar);				\
      YYPOPSTACK;						\
      goto yybackup;						\
    }								\
  else								\
    { yyerror ("syntax error: cannot back up"); YYERROR; }	\
while (0)

#define YYTERROR	1
#define YYERRCODE	256

#ifndef YYPURE
#define YYLEX		yylex()
#endif

#ifdef YYPURE
#ifdef YYLSP_NEEDED
#ifdef YYLEX_PARAM
#define YYLEX		yylex(&yylval, &yylloc, YYLEX_PARAM)
#else
#define YYLEX		yylex(&yylval, &yylloc)
#endif
#else /* not YYLSP_NEEDED */
#ifdef YYLEX_PARAM
#define YYLEX		yylex(&yylval, YYLEX_PARAM)
#else
#define YYLEX		yylex(&yylval)
#endif
#endif /* not YYLSP_NEEDED */
#endif

/* If nonreentrant, generate the variables here */

#ifndef YYPURE

int	yychar;			/*  the lookahead symbol		*/
YYSTYPE	yylval;			/*  the semantic value of the		*/
				/*  lookahead symbol			*/

#ifdef YYLSP_NEEDED
YYLTYPE yylloc;			/*  location data for the lookahead	*/
				/*  symbol				*/
#endif

int yynerrs;			/*  number of parse errors so far       */
#endif  /* not YYPURE */

#if YYDEBUG != 0
int yydebug;			/*  nonzero means print parse trace	*/
/* Since this is uninitialized, it does not stop multiple parsers
   from coexisting.  */
#endif

/*  YYINITDEPTH indicates the initial size of the parser's stacks	*/

#ifndef	YYINITDEPTH
#define YYINITDEPTH 200
#endif

/*  YYMAXDEPTH is the maximum size the stacks can grow to
    (effective only if the built-in stack extension method is used).  */

#if YYMAXDEPTH == 0
#undef YYMAXDEPTH
#endif

#ifndef YYMAXDEPTH
#define YYMAXDEPTH 10000
#endif

/* Prevent warning if -Wstrict-prototypes.  */
#ifdef __GNUC__
int yyparse (void);
#endif

#if __GNUC__ > 1		/* GNU C and GNU C++ define this.  */
#define __yy_memcpy(TO,FROM,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
#else				/* not GNU C or C++ */
#ifndef __cplusplus

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_memcpy (to, from, count)
     char *to;
     char *from;
     int count;
{
  register char *f = from;
  register char *t = to;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#else /* __cplusplus */

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_memcpy (char *to, char *from, int count)
{
  register char *f = from;
  register char *t = to;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#endif
#endif

#line 196 "/usr/cygnus/progressive-97r2/share/bison.simple"

/* The user can define YYPARSE_PARAM as the name of an argument to be passed
   into yyparse.  The argument should have type void *.
   It should actually point to an object.
   Grammar actions can access the variable by casting it
   to the proper pointer type.  */

#ifdef YYPARSE_PARAM
#ifdef __cplusplus
#define YYPARSE_PARAM_ARG void *YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else /* not __cplusplus */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL void *YYPARSE_PARAM;
#endif /* not __cplusplus */
#else /* not YYPARSE_PARAM */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif /* not YYPARSE_PARAM */

int
yyparse(YYPARSE_PARAM_ARG)
     YYPARSE_PARAM_DECL
{
  register int yystate;
  register int yyn;
  register short *yyssp;
  register YYSTYPE *yyvsp;
  int yyerrstatus;	/*  number of tokens to shift before error messages enabled */
  int yychar1 = 0;		/*  lookahead token as an internal (translated) token number */

  short	yyssa[YYINITDEPTH];	/*  the state stack			*/
  YYSTYPE yyvsa[YYINITDEPTH];	/*  the semantic value stack		*/

  short *yyss = yyssa;		/*  refer to the stacks thru separate pointers */
  YYSTYPE *yyvs = yyvsa;	/*  to allow yyoverflow to reallocate them elsewhere */

#ifdef YYLSP_NEEDED
  YYLTYPE yylsa[YYINITDEPTH];	/*  the location stack			*/
  YYLTYPE *yyls = yylsa;
  YYLTYPE *yylsp;

#define YYPOPSTACK   (yyvsp--, yyssp--, yylsp--)
#else
#define YYPOPSTACK   (yyvsp--, yyssp--)
#endif

  int yystacksize = YYINITDEPTH;

#ifdef YYPURE
  int yychar;
  YYSTYPE yylval;
  int yynerrs;
#ifdef YYLSP_NEEDED
  YYLTYPE yylloc;
#endif
#endif

  YYSTYPE yyval;		/*  the variable used to return		*/
				/*  semantic values from the action	*/
				/*  routines				*/

  int yylen;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Starting parse\n");
#endif

  yystate = 0;
  yyerrstatus = 0;
  yynerrs = 0;
  yychar = YYEMPTY;		/* Cause a token to be read.  */

  /* Initialize stack pointers.
     Waste one element of value and location stack
     so that they stay on the same level as the state stack.
     The wasted elements are never initialized.  */

  yyssp = yyss - 1;
  yyvsp = yyvs;
#ifdef YYLSP_NEEDED
  yylsp = yyls;
#endif

/* Push a new state, which is found in  yystate  .  */
/* In all cases, when you get here, the value and location stacks
   have just been pushed. so pushing a state here evens the stacks.  */
yynewstate:

  *++yyssp = yystate;

  if (yyssp >= yyss + yystacksize - 1)
    {
      /* Give user a chance to reallocate the stack */
      /* Use copies of these so that the &'s don't force the real ones into memory. */
      YYSTYPE *yyvs1 = yyvs;
      short *yyss1 = yyss;
#ifdef YYLSP_NEEDED
      YYLTYPE *yyls1 = yyls;
#endif

      /* Get the current used size of the three stacks, in elements.  */
      int size = yyssp - yyss + 1;

#ifdef yyoverflow
      /* Each stack pointer address is followed by the size of
	 the data in use in that stack, in bytes.  */
#ifdef YYLSP_NEEDED
      /* This used to be a conditional around just the two extra args,
	 but that might be undefined if yyoverflow is a macro.  */
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yyls1, size * sizeof (*yylsp),
		 &yystacksize);
#else
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yystacksize);
#endif

      yyss = yyss1; yyvs = yyvs1;
#ifdef YYLSP_NEEDED
      yyls = yyls1;
#endif
#else /* no yyoverflow */
      /* Extend the stack our own way.  */
      if (yystacksize >= YYMAXDEPTH)
	{
	  yyerror("parser stack overflow");
	  return 2;
	}
      yystacksize *= 2;
      if (yystacksize > YYMAXDEPTH)
	yystacksize = YYMAXDEPTH;
      yyss = (short *) alloca (yystacksize * sizeof (*yyssp));
      __yy_memcpy ((char *)yyss, (char *)yyss1, size * sizeof (*yyssp));
      yyvs = (YYSTYPE *) alloca (yystacksize * sizeof (*yyvsp));
      __yy_memcpy ((char *)yyvs, (char *)yyvs1, size * sizeof (*yyvsp));
#ifdef YYLSP_NEEDED
      yyls = (YYLTYPE *) alloca (yystacksize * sizeof (*yylsp));
      __yy_memcpy ((char *)yyls, (char *)yyls1, size * sizeof (*yylsp));
#endif
#endif /* no yyoverflow */

      yyssp = yyss + size - 1;
      yyvsp = yyvs + size - 1;
#ifdef YYLSP_NEEDED
      yylsp = yyls + size - 1;
#endif

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Stack size increased to %d\n", yystacksize);
#endif

      if (yyssp >= yyss + yystacksize - 1)
	YYABORT;
    }

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Entering state %d\n", yystate);
#endif

  goto yybackup;
 yybackup:

/* Do appropriate processing given the current state.  */
/* Read a lookahead token if we need one and don't already have one.  */
/* yyresume: */

  /* First try to decide what to do without reference to lookahead token.  */

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* yychar is either YYEMPTY or YYEOF
     or a valid token in external form.  */

  if (yychar == YYEMPTY)
    {
#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Reading a token: ");
#endif
      yychar = YYLEX;
    }

  /* Convert token to internal form (in yychar1) for indexing tables with */

  if (yychar <= 0)		/* This means end of input. */
    {
      yychar1 = 0;
      yychar = YYEOF;		/* Don't call YYLEX any more */

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Now at end of input.\n");
#endif
    }
  else
    {
      yychar1 = YYTRANSLATE(yychar);

#if YYDEBUG != 0
      if (yydebug)
	{
	  fprintf (stderr, "Next token is %d (%s", yychar, yytname[yychar1]);
	  /* Give the individual parser a way to print the precise meaning
	     of a token, for further debugging info.  */
#ifdef YYPRINT
	  YYPRINT (stderr, yychar, yylval);
#endif
	  fprintf (stderr, ")\n");
	}
#endif
    }

  yyn += yychar1;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != yychar1)
    goto yydefault;

  yyn = yytable[yyn];

  /* yyn is what to do for this token type in this state.
     Negative => reduce, -yyn is rule number.
     Positive => shift, yyn is new state.
       New state is final state => don't bother to shift,
       just return success.
     0, or most negative number => error.  */

  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrlab;

  if (yyn == YYFINAL)
    YYACCEPT;

  /* Shift the lookahead token.  */

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting token %d (%s), ", yychar, yytname[yychar1]);
#endif

  /* Discard the token being shifted unless it is eof.  */
  if (yychar != YYEOF)
    yychar = YYEMPTY;

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  /* count tokens shifted since error; after three, turn off error status.  */
  if (yyerrstatus) yyerrstatus--;

  yystate = yyn;
  goto yynewstate;

/* Do the default action for the current state.  */
yydefault:

  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;

/* Do a reduction.  yyn is the number of a rule to reduce with.  */
yyreduce:
  yylen = yyr2[yyn];
  if (yylen > 0)
    yyval = yyvsp[1-yylen]; /* implement default value of the action */

#if YYDEBUG != 0
  if (yydebug)
    {
      int i;

      fprintf (stderr, "Reducing via rule %d (line %d), ",
	       yyn, yyrline[yyn]);

      /* Print the symbols being reduced, and their result.  */
      for (i = yyprhs[yyn]; yyrhs[i] > 0; i++)
	fprintf (stderr, "%s ", yytname[yyrhs[i]]);
      fprintf (stderr, " -> %s\n", yytname[yyr1[yyn]]);
    }
#endif


  switch (yyn) {

case 16:
#line 174 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    rcparse_discard_strings ();
	  ;
    break;}
case 17:
#line 183 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_accelerator (yyvsp[-5].id, &yyvsp[-3].res_info, yyvsp[-1].pacc);
	  ;
    break;}
case 18:
#line 190 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.pacc = NULL;
	  ;
    break;}
case 19:
#line 194 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    struct accelerator *a;

	    a = (struct accelerator *) res_alloc (sizeof *a);
	    *a = yyvsp[0].acc;
	    if (yyvsp[-1].pacc == NULL)
	      yyval.pacc = a;
	    else
	      {
		struct accelerator **pp;

		for (pp = &yyvsp[-1].pacc->next; *pp != NULL; pp = &(*pp)->next)
		  ;
		*pp = a;
		yyval.pacc = yyvsp[-1].pacc;
	      }
	  ;
    break;}
case 20:
#line 215 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.acc = yyvsp[-1].acc;
	    yyval.acc.id = yyvsp[0].il;
	  ;
    break;}
case 21:
#line 220 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.acc = yyvsp[-3].acc;
	    yyval.acc.id = yyvsp[-2].il;
	    yyval.acc.flags |= yyvsp[0].is;
	    if ((yyval.acc.flags & ACC_VIRTKEY) == 0
		&& (yyval.acc.flags & (ACC_SHIFT | ACC_CONTROL | ACC_ALT)) != 0)
	      rcparse_warning ("inappropriate modifiers for non-VIRTKEY");
	  ;
    break;}
case 22:
#line 232 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    const char *s = yyvsp[0].s;
	    char ch;

	    yyval.acc.next = NULL;
	    yyval.acc.id = 0;
	    ch = *s;
	    if (ch != '^')
	      yyval.acc.flags = 0;
	    else
	      {
		yyval.acc.flags = ACC_CONTROL | ACC_VIRTKEY;
		++s;
		ch = *s;
		ch = toupper ((unsigned char) ch);
	      }
	    yyval.acc.key = ch;
	    if (s[1] != '\0')
	      rcparse_warning ("accelerator should only be one character");
	  ;
    break;}
case 23:
#line 253 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.acc.next = NULL;
	    yyval.acc.flags = 0;
	    yyval.acc.id = 0;
	    yyval.acc.key = yyvsp[0].il;
	  ;
    break;}
case 24:
#line 263 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = yyvsp[0].is;
	  ;
    break;}
case 25:
#line 267 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = yyvsp[-2].is | yyvsp[0].is;
	  ;
    break;}
case 26:
#line 272 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = yyvsp[-1].is | yyvsp[0].is;
	  ;
    break;}
case 27:
#line 279 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = ACC_VIRTKEY;
	  ;
    break;}
case 28:
#line 283 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    /* This is just the absence of VIRTKEY.  */
	    yyval.is = 0;
	  ;
    break;}
case 29:
#line 288 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = ACC_NOINVERT;
	  ;
    break;}
case 30:
#line 292 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = ACC_SHIFT;
	  ;
    break;}
case 31:
#line 296 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = ACC_CONTROL;
	  ;
    break;}
case 32:
#line 300 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = ACC_ALT;
	  ;
    break;}
case 33:
#line 309 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_bitmap (yyvsp[-3].id, &yyvsp[-1].res_info, yyvsp[0].s);
	  ;
    break;}
case 34:
#line 318 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_cursor (yyvsp[-3].id, &yyvsp[-1].res_info, yyvsp[0].s);
	  ;
    break;}
case 35:
#line 328 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      memset (&dialog, 0, sizeof dialog);
	      dialog.x = yyvsp[-3].il;
	      dialog.y = yyvsp[-2].il;
	      dialog.width = yyvsp[-1].il;
	      dialog.height = yyvsp[0].il;
	      dialog.style = WS_POPUP | WS_BORDER | WS_SYSMENU;
	      dialog.exstyle = yyvsp[-4].il;
	      dialog.menu.named = 1;
	      dialog.class.named = 1;
	      dialog.font = NULL;
	      dialog.ex = NULL;
	      dialog.controls = NULL;
	      sub_res_info = yyvsp[-5].res_info;
	    ;
    break;}
case 36:
#line 344 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_dialog (yyvsp[-12].id, &sub_res_info, &dialog);
	  ;
    break;}
case 37:
#line 349 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      memset (&dialog, 0, sizeof dialog);
	      dialog.x = yyvsp[-3].il;
	      dialog.y = yyvsp[-2].il;
	      dialog.width = yyvsp[-1].il;
	      dialog.height = yyvsp[0].il;
	      dialog.style = WS_POPUP | WS_BORDER | WS_SYSMENU;
	      dialog.exstyle = yyvsp[-4].il;
	      dialog.menu.named = 1;
	      dialog.class.named = 1;
	      dialog.font = NULL;
	      dialog.ex = ((struct dialog_ex *)
			   res_alloc (sizeof (struct dialog_ex)));
	      memset (dialog.ex, 0, sizeof (struct dialog_ex));
	      dialog.controls = NULL;
	      sub_res_info = yyvsp[-5].res_info;
	    ;
    break;}
case 38:
#line 367 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_dialog (yyvsp[-12].id, &sub_res_info, &dialog);
	  ;
    break;}
case 39:
#line 372 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      memset (&dialog, 0, sizeof dialog);
	      dialog.x = yyvsp[-4].il;
	      dialog.y = yyvsp[-3].il;
	      dialog.width = yyvsp[-2].il;
	      dialog.height = yyvsp[-1].il;
	      dialog.style = WS_POPUP | WS_BORDER | WS_SYSMENU;
	      dialog.exstyle = yyvsp[-5].il;
	      dialog.menu.named = 1;
	      dialog.class.named = 1;
	      dialog.font = NULL;
	      dialog.ex = ((struct dialog_ex *)
			   res_alloc (sizeof (struct dialog_ex)));
	      memset (dialog.ex, 0, sizeof (struct dialog_ex));
	      dialog.ex->help = yyvsp[0].il;
	      dialog.controls = NULL;
	      sub_res_info = yyvsp[-6].res_info;
	    ;
    break;}
case 40:
#line 391 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_dialog (yyvsp[-13].id, &sub_res_info, &dialog);
	  ;
    break;}
case 41:
#line 398 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.il = 0;
	  ;
    break;}
case 42:
#line 402 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.il = yyvsp[0].il;
	  ;
    break;}
case 44:
#line 410 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    unicode_from_ascii ((int *) NULL, &dialog.caption, yyvsp[0].s);
	  ;
    break;}
case 45:
#line 414 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    dialog.class = yyvsp[0].id;
	  ;
    break;}
case 46:
#line 418 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{ style = dialog.style; ;
    break;}
case 47:
#line 420 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    dialog.style = style;
	  ;
    break;}
case 48:
#line 424 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    dialog.exstyle = yyvsp[0].il;
	  ;
    break;}
case 49:
#line 428 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    dialog.style |= DS_SETFONT;
	    dialog.pointsize = yyvsp[-2].il;
	    unicode_from_ascii ((int *) NULL, &dialog.font, yyvsp[0].s);
	  ;
    break;}
case 50:
#line 434 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    dialog.style |= DS_SETFONT;
	    dialog.pointsize = yyvsp[-4].il;
	    unicode_from_ascii ((int *) NULL, &dialog.font, yyvsp[-2].s);
	    if (dialog.ex == NULL)
	      rcparse_warning ("extended FONT requires DIALOGEX");
	    else
	      {
		dialog.ex->weight = yyvsp[-1].il;
		dialog.ex->italic = yyvsp[0].il;
	      }
	  ;
    break;}
case 51:
#line 447 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    dialog.menu = yyvsp[0].id;
	  ;
    break;}
case 52:
#line 451 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    sub_res_info.characteristics = yyvsp[0].il;
	  ;
    break;}
case 53:
#line 455 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    sub_res_info.language = yyvsp[-1].il | (yyvsp[0].il << 8);
	  ;
    break;}
case 54:
#line 459 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    sub_res_info.version = yyvsp[0].il;
	  ;
    break;}
case 56:
#line 467 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    struct dialog_control **pp;

	    for (pp = &dialog.controls; *pp != NULL; pp = &(*pp)->next)
	      ;
	    *pp = yyvsp[0].dialog_control;
	  ;
    break;}
case 57:
#line 478 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = BS_AUTO3STATE | WS_TABSTOP;
	      base_style = BS_AUTO3STATE;
	      class = CTL_BUTTON;
	    ;
    break;}
case 58:
#line 484 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 59:
#line 488 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = BS_AUTOCHECKBOX | WS_TABSTOP;
	      base_style = BS_AUTOCHECKBOX;
	      class = CTL_BUTTON;
	    ;
    break;}
case 60:
#line 494 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 61:
#line 498 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = BS_AUTORADIOBUTTON | WS_TABSTOP;
	      base_style = BS_AUTORADIOBUTTON;
	      class = CTL_BUTTON;
	    ;
    break;}
case 62:
#line 504 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 63:
#line 508 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = ES_LEFT | WS_BORDER | WS_TABSTOP;
	      base_style = ES_LEFT | WS_BORDER | WS_TABSTOP;
	      class = CTL_EDIT;
	    ;
    break;}
case 64:
#line 514 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	    if (dialog.ex == NULL)
	      rcparse_warning ("IEDIT requires DIALOGEX");
	    res_string_to_id (&yyval.dialog_control->class, "BEDIT");
	  ;
    break;}
case 65:
#line 521 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = BS_CHECKBOX | WS_TABSTOP;
	      base_style = BS_CHECKBOX | WS_TABSTOP;
	      class = CTL_BUTTON;
	    ;
    break;}
case 66:
#line 527 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 67:
#line 531 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = CBS_SIMPLE | WS_TABSTOP;
	      base_style = 0;
	      class = CTL_COMBOBOX;
	    ;
    break;}
case 68:
#line 537 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 69:
#line 542 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = define_control (yyvsp[-9].s, yyvsp[-8].il, yyvsp[-5].il, yyvsp[-4].il, yyvsp[-3].il, yyvsp[-2].il, yyvsp[-7].il, style, yyvsp[-1].il);
	    if (yyvsp[0].rcdata_item != NULL)
	      {
		if (dialog.ex == NULL)
		  rcparse_warning ("control data requires DIALOGEX");
		yyval.dialog_control->data = yyvsp[0].rcdata_item;
	      }
	  ;
    break;}
case 70:
#line 553 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = define_control (yyvsp[-10].s, yyvsp[-9].il, yyvsp[-6].il, yyvsp[-5].il, yyvsp[-4].il, yyvsp[-3].il, yyvsp[-8].il, style, yyvsp[-2].il);
	    if (dialog.ex == NULL)
	      rcparse_warning ("help ID requires DIALOGEX");
	    yyval.dialog_control->help = yyvsp[-1].il;
	    yyval.dialog_control->data = yyvsp[0].rcdata_item;
	  ;
    break;}
case 71:
#line 561 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = SS_CENTER | WS_GROUP;
	      base_style = SS_CENTER;
	      class = CTL_STATIC;
	    ;
    break;}
case 72:
#line 567 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 73:
#line 571 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = BS_DEFPUSHBUTTON | WS_TABSTOP;
	      base_style = BS_DEFPUSHBUTTON | WS_TABSTOP;
	      class = CTL_BUTTON;
	    ;
    break;}
case 74:
#line 577 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 75:
#line 581 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = ES_LEFT | WS_BORDER | WS_TABSTOP;
	      base_style = ES_LEFT | WS_BORDER | WS_TABSTOP;
	      class = CTL_EDIT;
	    ;
    break;}
case 76:
#line 587 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 77:
#line 591 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = BS_GROUPBOX;
	      base_style = BS_GROUPBOX;
	      class = CTL_BUTTON;
	    ;
    break;}
case 78:
#line 597 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 79:
#line 601 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = ES_LEFT | WS_BORDER | WS_TABSTOP;
	      base_style = ES_LEFT | WS_BORDER | WS_TABSTOP;
	      class = CTL_EDIT;
	    ;
    break;}
case 80:
#line 607 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	    if (dialog.ex == NULL)
	      rcparse_warning ("IEDIT requires DIALOGEX");
	    res_string_to_id (&yyval.dialog_control->class, "HEDIT");
	  ;
    break;}
case 81:
#line 614 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = define_control (yyvsp[-4].s, yyvsp[-3].il, yyvsp[-2].il, yyvsp[-1].il, 0, 0, CTL_STATIC,
				 SS_ICON | WS_CHILD | WS_VISIBLE, 0);
	    if (yyvsp[0].rcdata_item != NULL)
	      {
		if (dialog.ex == NULL)
		  rcparse_warning ("control data requires DIALOGEX");
		yyval.dialog_control->data = yyvsp[0].rcdata_item;
	      }
	  ;
    break;}
case 82:
#line 626 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
    	    yyval.dialog_control = define_control (yyvsp[-8].s, yyvsp[-7].il, yyvsp[-6].il, yyvsp[-5].il, yyvsp[-4].il, yyvsp[-3].il, CTL_STATIC,
				 style, yyvsp[-1].il);
	    if (yyvsp[0].rcdata_item != NULL)
	      {
		if (dialog.ex == NULL)
		  rcparse_warning ("control data requires DIALOGEX");
		yyval.dialog_control->data = yyvsp[0].rcdata_item;
	      }
	  ;
    break;}
case 83:
#line 638 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
    	    yyval.dialog_control = define_control (yyvsp[-9].s, yyvsp[-8].il, yyvsp[-7].il, yyvsp[-6].il, yyvsp[-5].il, yyvsp[-4].il, CTL_STATIC,
				 style, yyvsp[-2].il);
	    if (dialog.ex == NULL)
	      rcparse_warning ("help ID requires DIALOGEX");
	    yyval.dialog_control->help = yyvsp[-1].il;
	    yyval.dialog_control->data = yyvsp[0].rcdata_item;
	  ;
    break;}
case 84:
#line 647 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = ES_LEFT | WS_BORDER | WS_TABSTOP;
	      base_style = ES_LEFT | WS_BORDER | WS_TABSTOP;
	      class = CTL_EDIT;
	    ;
    break;}
case 85:
#line 653 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	    if (dialog.ex == NULL)
	      rcparse_warning ("IEDIT requires DIALOGEX");
	    res_string_to_id (&yyval.dialog_control->class, "IEDIT");
	  ;
    break;}
case 86:
#line 660 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = LBS_NOTIFY | WS_BORDER;
	      base_style = LBS_NOTIFY | WS_BORDER;
	      class = CTL_LISTBOX;
	    ;
    break;}
case 87:
#line 666 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 88:
#line 670 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = SS_LEFT | WS_GROUP;
	      base_style = SS_LEFT;
	      class = CTL_STATIC;
	    ;
    break;}
case 89:
#line 676 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 90:
#line 680 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = BS_PUSHBOX | WS_TABSTOP;
	      base_style = BS_PUSHBOX;
	      class = CTL_BUTTON;
	    ;
    break;}
case 91:
#line 686 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 92:
#line 690 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = BS_PUSHBUTTON | WS_TABSTOP;
	      base_style = BS_PUSHBUTTON | WS_TABSTOP;
	      class = CTL_BUTTON;
	    ;
    break;}
case 93:
#line 696 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 94:
#line 700 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = BS_RADIOBUTTON | WS_TABSTOP;
	      base_style = BS_RADIOBUTTON;
	      class = CTL_BUTTON;
	    ;
    break;}
case 95:
#line 706 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 96:
#line 710 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = SS_RIGHT | WS_GROUP;
	      base_style = SS_RIGHT;
	      class = CTL_STATIC;
	    ;
    break;}
case 97:
#line 716 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 98:
#line 720 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = SBS_HORZ;
	      base_style = 0;
	      class = CTL_SCROLLBAR;
	    ;
    break;}
case 99:
#line 726 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 100:
#line 730 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	      default_style = BS_3STATE | WS_TABSTOP;
	      base_style = BS_3STATE;
	      class = CTL_BUTTON;
	    ;
    break;}
case 101:
#line 736 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = yyvsp[0].dialog_control;
	  ;
    break;}
case 102:
#line 741 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{ style = WS_CHILD | WS_VISIBLE; ;
    break;}
case 103:
#line 743 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = define_control (yyvsp[-14].s, yyvsp[-12].il, yyvsp[-10].il, yyvsp[-8].il, yyvsp[-6].il, yyvsp[-4].il, CTL_BUTTON,
				 style, yyvsp[0].il);
	  ;
    break;}
case 104:
#line 759 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = define_control (yyvsp[-6].s, yyvsp[-5].il, yyvsp[-4].il, yyvsp[-3].il, yyvsp[-2].il, yyvsp[-1].il, class,
				 default_style | WS_CHILD | WS_VISIBLE, 0);
	    if (yyvsp[0].rcdata_item != NULL)
	      {
		if (dialog.ex == NULL)
		  rcparse_warning ("control data requires DIALOGEX");
		yyval.dialog_control->data = yyvsp[0].rcdata_item;
	      }
	  ;
    break;}
case 105:
#line 771 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = define_control (yyvsp[-8].s, yyvsp[-7].il, yyvsp[-6].il, yyvsp[-5].il, yyvsp[-4].il, yyvsp[-3].il, class, style, yyvsp[-1].il);
	    if (yyvsp[0].rcdata_item != NULL)
	      {
		if (dialog.ex == NULL)
		  rcparse_warning ("control data requires DIALOGEX");
		yyval.dialog_control->data = yyvsp[0].rcdata_item;
	      }
	  ;
    break;}
case 106:
#line 782 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.dialog_control = define_control (yyvsp[-9].s, yyvsp[-8].il, yyvsp[-7].il, yyvsp[-6].il, yyvsp[-5].il, yyvsp[-4].il, class, style, yyvsp[-2].il);
	    if (dialog.ex == NULL)
	      rcparse_warning ("help ID requires DIALOGEX");
	    yyval.dialog_control->help = yyvsp[-1].il;
	    yyval.dialog_control->data = yyvsp[0].rcdata_item;
	  ;
    break;}
case 107:
#line 793 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.s = NULL;
	  ;
    break;}
case 108:
#line 797 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.s = yyvsp[-1].s;
	  ;
    break;}
case 109:
#line 804 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.rcdata_item = NULL;
	  ;
    break;}
case 110:
#line 808 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.rcdata_item = yyvsp[-1].rcdata.first;
	  ;
    break;}
case 111:
#line 817 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{ style = WS_CHILD | WS_VISIBLE; ;
    break;}
case 113:
#line 823 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{ style = SS_ICON | WS_CHILD | WS_VISIBLE; ;
    break;}
case 115:
#line 829 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{ style = base_style | WS_CHILD | WS_VISIBLE; ;
    break;}
case 117:
#line 837 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_font (yyvsp[-3].id, &yyvsp[-1].res_info, yyvsp[0].s);
	  ;
    break;}
case 118:
#line 846 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_icon (yyvsp[-3].id, &yyvsp[-1].res_info, yyvsp[0].s);
	  ;
    break;}
case 119:
#line 856 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    language = yyvsp[-1].il | (yyvsp[0].il << 8);
	  ;
    break;}
case 120:
#line 865 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_menu (yyvsp[-5].id, &yyvsp[-3].res_info, yyvsp[-1].menuitem);
	  ;
    break;}
case 121:
#line 872 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.menuitem = NULL;
	  ;
    break;}
case 122:
#line 876 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    if (yyvsp[-1].menuitem == NULL)
	      yyval.menuitem = yyvsp[0].menuitem;
	    else
	      {
		struct menuitem **pp;

		for (pp = &yyvsp[-1].menuitem->next; *pp != NULL; pp = &(*pp)->next)
		  ;
		*pp = yyvsp[0].menuitem;
		yyval.menuitem = yyvsp[-1].menuitem;
	      }
	  ;
    break;}
case 123:
#line 893 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.menuitem = define_menuitem (yyvsp[-2].s, yyvsp[-1].il, yyvsp[0].is, 0, 0, NULL);
	  ;
    break;}
case 124:
#line 897 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.menuitem = define_menuitem (NULL, 0, 0, 0, 0, NULL);
	  ;
    break;}
case 125:
#line 901 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.menuitem = define_menuitem (yyvsp[-4].s, 0, yyvsp[-3].is, 0, 0, yyvsp[-1].menuitem);
	  ;
    break;}
case 126:
#line 908 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = 0;
	  ;
    break;}
case 127:
#line 912 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = yyvsp[-2].is | yyvsp[0].is;
	  ;
    break;}
case 128:
#line 916 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = yyvsp[-1].is | yyvsp[0].is;
	  ;
    break;}
case 129:
#line 923 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = MENUITEM_CHECKED;
	  ;
    break;}
case 130:
#line 927 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = MENUITEM_GRAYED;
	  ;
    break;}
case 131:
#line 931 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = MENUITEM_HELP;
	  ;
    break;}
case 132:
#line 935 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = MENUITEM_INACTIVE;
	  ;
    break;}
case 133:
#line 939 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = MENUITEM_MENUBARBREAK;
	  ;
    break;}
case 134:
#line 943 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.is = MENUITEM_MENUBREAK;
	  ;
    break;}
case 135:
#line 952 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_menu (yyvsp[-5].id, &yyvsp[-3].res_info, yyvsp[-1].menuitem);
	  ;
    break;}
case 136:
#line 959 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.menuitem = NULL;
	  ;
    break;}
case 137:
#line 963 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    if (yyvsp[-1].menuitem == NULL)
	      yyval.menuitem = yyvsp[0].menuitem;
	    else
	      {
		struct menuitem **pp;

		for (pp = &yyvsp[-1].menuitem->next; *pp != NULL; pp = &(*pp)->next)
		  ;
		*pp = yyvsp[0].menuitem;
		yyval.menuitem = yyvsp[-1].menuitem;
	      }
	  ;
    break;}
case 138:
#line 980 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.menuitem = define_menuitem (yyvsp[0].s, 0, 0, 0, 0, NULL);
	  ;
    break;}
case 139:
#line 984 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.menuitem = define_menuitem (yyvsp[-1].s, yyvsp[0].il, 0, 0, 0, NULL);
	  ;
    break;}
case 140:
#line 988 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.menuitem = define_menuitem (yyvsp[-3].s, yyvsp[-2].il, yyvsp[-1].il, yyvsp[0].il, 0, NULL);
	  ;
    break;}
case 141:
#line 992 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.menuitem = define_menuitem (yyvsp[-3].s, 0, 0, 0, 0, yyvsp[-1].menuitem);
	  ;
    break;}
case 142:
#line 996 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.menuitem = define_menuitem (yyvsp[-4].s, yyvsp[-3].il, 0, 0, 0, yyvsp[-1].menuitem);
	  ;
    break;}
case 143:
#line 1000 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.menuitem = define_menuitem (yyvsp[-5].s, yyvsp[-4].il, yyvsp[-3].il, 0, 0, yyvsp[-1].menuitem);
	  ;
    break;}
case 144:
#line 1005 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.menuitem = define_menuitem (yyvsp[-7].s, yyvsp[-6].il, yyvsp[-5].il, yyvsp[-4].il, yyvsp[-3].il, yyvsp[-1].menuitem);
	  ;
    break;}
case 145:
#line 1014 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_messagetable (yyvsp[-3].id, &yyvsp[-1].res_info, yyvsp[0].s);
	  ;
    break;}
case 146:
#line 1023 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_rcdata (yyvsp[-5].id, &yyvsp[-3].res_info, yyvsp[-1].rcdata.first);
	  ;
    break;}
case 147:
#line 1032 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    rcparse_rcdata ();
	  ;
    break;}
case 148:
#line 1036 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    rcparse_normal ();
	    yyval.rcdata = yyvsp[0].rcdata;
	  ;
    break;}
case 149:
#line 1044 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.rcdata.first = NULL;
	    yyval.rcdata.last = NULL;
	  ;
    break;}
case 150:
#line 1049 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.rcdata = yyvsp[0].rcdata;
	  ;
    break;}
case 151:
#line 1056 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    struct rcdata_item *ri;

	    ri = define_rcdata_string (yyvsp[0].ss.s, yyvsp[0].ss.length);
	    yyval.rcdata.first = ri;
	    yyval.rcdata.last = ri;
	  ;
    break;}
case 152:
#line 1064 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    struct rcdata_item *ri;

	    ri = define_rcdata_number (yyvsp[0].i.val, yyvsp[0].i.dword);
	    yyval.rcdata.first = ri;
	    yyval.rcdata.last = ri;
	  ;
    break;}
case 153:
#line 1072 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    struct rcdata_item *ri;

	    ri = define_rcdata_string (yyvsp[0].ss.s, yyvsp[0].ss.length);
	    yyval.rcdata.first = yyvsp[-2].rcdata.first;
	    yyvsp[-2].rcdata.last->next = ri;
	    yyval.rcdata.last = ri;
	  ;
    break;}
case 154:
#line 1081 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    struct rcdata_item *ri;

	    ri = define_rcdata_number (yyvsp[0].i.val, yyvsp[0].i.dword);
	    yyval.rcdata.first = yyvsp[-2].rcdata.first;
	    yyvsp[-2].rcdata.last->next = ri;
	    yyval.rcdata.last = ri;
	  ;
    break;}
case 155:
#line 1095 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{ sub_res_info = yyvsp[-1].res_info; ;
    break;}
case 158:
#line 1102 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_stringtable (&sub_res_info, yyvsp[-1].il, yyvsp[0].s);
	  ;
    break;}
case 159:
#line 1106 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_stringtable (&sub_res_info, yyvsp[-2].il, yyvsp[0].s);
	  ;
    break;}
case 160:
#line 1116 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_user_data (yyvsp[-5].id, yyvsp[-4].id, &yyvsp[-3].res_info, yyvsp[-1].rcdata.first);
	  ;
    break;}
case 161:
#line 1120 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_user_file (yyvsp[-3].id, yyvsp[-2].id, &yyvsp[-1].res_info, yyvsp[0].s);
	  ;
    break;}
case 162:
#line 1129 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    define_versioninfo (yyvsp[-5].id, language, yyvsp[-3].fixver, yyvsp[-1].verinfo);
	  ;
    break;}
case 163:
#line 1136 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.fixver = ((struct fixed_versioninfo *)
		  res_alloc (sizeof (struct fixed_versioninfo)));
	    memset (yyval.fixver, 0, sizeof (struct fixed_versioninfo));
	  ;
    break;}
case 164:
#line 1142 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyvsp[-5].fixver->file_version_ms = (yyvsp[-3].il << 16) | yyvsp[-2].il;
	    yyvsp[-5].fixver->file_version_ls = (yyvsp[-1].il << 16) | yyvsp[0].il;
	    yyval.fixver = yyvsp[-5].fixver;
	  ;
    break;}
case 165:
#line 1148 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyvsp[-5].fixver->product_version_ms = (yyvsp[-3].il << 16) | yyvsp[-2].il;
	    yyvsp[-5].fixver->product_version_ls = (yyvsp[-1].il << 16) | yyvsp[0].il;
	    yyval.fixver = yyvsp[-5].fixver;
	  ;
    break;}
case 166:
#line 1154 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyvsp[-2].fixver->file_flags_mask = yyvsp[0].il;
	    yyval.fixver = yyvsp[-2].fixver;
	  ;
    break;}
case 167:
#line 1159 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyvsp[-2].fixver->file_flags = yyvsp[0].il;
	    yyval.fixver = yyvsp[-2].fixver;
	  ;
    break;}
case 168:
#line 1164 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyvsp[-2].fixver->file_os = yyvsp[0].il;
	    yyval.fixver = yyvsp[-2].fixver;
	  ;
    break;}
case 169:
#line 1169 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyvsp[-2].fixver->file_type = yyvsp[0].il;
	    yyval.fixver = yyvsp[-2].fixver;
	  ;
    break;}
case 170:
#line 1174 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyvsp[-2].fixver->file_subtype = yyvsp[0].il;
	    yyval.fixver = yyvsp[-2].fixver;
	  ;
    break;}
case 171:
#line 1188 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.verinfo = NULL;
	  ;
    break;}
case 172:
#line 1192 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.verinfo = append_ver_stringfileinfo (yyvsp[-7].verinfo, yyvsp[-4].s, yyvsp[-2].verstring);
	  ;
    break;}
case 173:
#line 1196 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.verinfo = append_ver_varfileinfo (yyvsp[-6].verinfo, yyvsp[-2].s, yyvsp[-1].vervar);
	  ;
    break;}
case 174:
#line 1203 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.verstring = NULL;
	  ;
    break;}
case 175:
#line 1207 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.verstring = append_verval (yyvsp[-4].verstring, yyvsp[-2].s, yyvsp[0].s);
	  ;
    break;}
case 176:
#line 1214 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.vervar = NULL;
	  ;
    break;}
case 177:
#line 1218 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.vervar = append_vertrans (yyvsp[-2].vervar, yyvsp[-1].il, yyvsp[0].il);
	  ;
    break;}
case 178:
#line 1227 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.id.named = 0;
	    yyval.id.u.id = yyvsp[0].il;
	  ;
    break;}
case 179:
#line 1232 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    char *copy, *s;

	    /* It seems that resource ID's are forced to upper case.  */
	    copy = xstrdup (yyvsp[0].s);
	    for (s = copy; *s != '\0'; s++)
	      if (islower (*s))
		*s = toupper (*s);
	    res_string_to_id (&yyval.id, copy);
	    free (copy);
	  ;
    break;}
case 180:
#line 1250 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    memset (&yyval.res_info, 0, sizeof (struct res_res_info));
	    yyval.res_info.language = language;
	    /* FIXME: Is this the right default?  */
	    yyval.res_info.memflags = MEMFLAG_MOVEABLE;
	  ;
    break;}
case 181:
#line 1257 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.res_info = yyvsp[-1].res_info;
	    yyval.res_info.memflags |= yyvsp[0].memflags.on;
	    yyval.res_info.memflags &=~ yyvsp[0].memflags.off;
	  ;
    break;}
case 182:
#line 1263 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.res_info = yyvsp[-2].res_info;
	    yyval.res_info.characteristics = yyvsp[0].il;
	  ;
    break;}
case 183:
#line 1268 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.res_info = yyvsp[-3].res_info;
	    yyval.res_info.language = yyvsp[-1].il | (yyvsp[0].il << 8);
	  ;
    break;}
case 184:
#line 1273 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.res_info = yyvsp[-2].res_info;
	    yyval.res_info.version = yyvsp[0].il;
	  ;
    break;}
case 185:
#line 1283 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    memset (&yyval.res_info, 0, sizeof (struct res_res_info));
	    yyval.res_info.language = language;
	    yyval.res_info.memflags = MEMFLAG_MOVEABLE | MEMFLAG_DISCARDABLE;
	  ;
    break;}
case 186:
#line 1289 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.res_info = yyvsp[-1].res_info;
	    yyval.res_info.memflags |= yyvsp[0].memflags.on;
	    yyval.res_info.memflags &=~ yyvsp[0].memflags.off;
	  ;
    break;}
case 187:
#line 1300 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    memset (&yyval.res_info, 0, sizeof (struct res_res_info));
	    yyval.res_info.language = language;
	    yyval.res_info.memflags = MEMFLAG_MOVEABLE;
	  ;
    break;}
case 188:
#line 1306 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.res_info = yyvsp[-1].res_info;
	    yyval.res_info.memflags |= yyvsp[0].memflags.on;
	    yyval.res_info.memflags &=~ yyvsp[0].memflags.off;
	  ;
    break;}
case 189:
#line 1318 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.memflags.on = MEMFLAG_MOVEABLE;
	    yyval.memflags.off = 0;
	  ;
    break;}
case 190:
#line 1323 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.memflags.on = 0;
	    yyval.memflags.off = MEMFLAG_MOVEABLE;
	  ;
    break;}
case 191:
#line 1328 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.memflags.on = MEMFLAG_PURE;
	    yyval.memflags.off = 0;
	  ;
    break;}
case 192:
#line 1333 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.memflags.on = 0;
	    yyval.memflags.off = MEMFLAG_PURE;
	  ;
    break;}
case 193:
#line 1338 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.memflags.on = MEMFLAG_PRELOAD;
	    yyval.memflags.off = 0;
	  ;
    break;}
case 194:
#line 1343 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.memflags.on = 0;
	    yyval.memflags.off = MEMFLAG_PRELOAD;
	  ;
    break;}
case 195:
#line 1348 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.memflags.on = MEMFLAG_DISCARDABLE;
	    yyval.memflags.off = 0;
	  ;
    break;}
case 196:
#line 1358 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.s = yyvsp[0].s;
	  ;
    break;}
case 197:
#line 1362 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.s = yyvsp[0].s;
	  ;
    break;}
case 198:
#line 1379 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    style |= yyvsp[0].il;
	  ;
    break;}
case 199:
#line 1383 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    style &=~ yyvsp[0].il;
	  ;
    break;}
case 200:
#line 1387 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    style |= yyvsp[0].il;
	  ;
    break;}
case 201:
#line 1391 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    style &=~ yyvsp[0].il;
	  ;
    break;}
case 202:
#line 1398 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.il = yyvsp[0].i.val;
	  ;
    break;}
case 203:
#line 1402 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.il = yyvsp[-1].il;
	  ;
    break;}
case 204:
#line 1411 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.il = 0;
	  ;
    break;}
case 205:
#line 1415 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.il = yyvsp[0].il;
	  ;
    break;}
case 206:
#line 1424 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.il = yyvsp[0].il;
	  ;
    break;}
case 207:
#line 1433 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.il = yyvsp[0].i.val;
	  ;
    break;}
case 208:
#line 1442 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i = yyvsp[0].i;
	  ;
    break;}
case 209:
#line 1446 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i = yyvsp[-1].i;
	  ;
    break;}
case 210:
#line 1450 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = ~ yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[0].i.dword;
	  ;
    break;}
case 211:
#line 1455 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = - yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[0].i.dword;
	  ;
    break;}
case 212:
#line 1460 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val * yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 213:
#line 1465 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val / yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 214:
#line 1470 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val % yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 215:
#line 1475 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val + yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 216:
#line 1480 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val - yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 217:
#line 1485 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val & yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 218:
#line 1490 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val ^ yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 219:
#line 1495 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val | yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 220:
#line 1506 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.il = yyvsp[0].il;
	  ;
    break;}
case 221:
#line 1515 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.il = yyvsp[0].i.val;
	  ;
    break;}
case 222:
#line 1526 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i = yyvsp[0].i;
	  ;
    break;}
case 223:
#line 1530 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i = yyvsp[-1].i;
	  ;
    break;}
case 224:
#line 1534 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = ~ yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[0].i.dword;
	  ;
    break;}
case 225:
#line 1539 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val * yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 226:
#line 1544 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val / yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 227:
#line 1549 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val % yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 228:
#line 1554 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val + yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 229:
#line 1559 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val - yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 230:
#line 1564 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val & yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 231:
#line 1569 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val ^ yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
case 232:
#line 1574 "/5g/ian/binutils/release/copy/binutils/rcparse.y"
{
	    yyval.i.val = yyvsp[-2].i.val | yyvsp[0].i.val;
	    yyval.i.dword = yyvsp[-2].i.dword || yyvsp[0].i.dword;
	  ;
    break;}
}
   /* the action file gets copied in in place of this dollarsign */
#line 498 "/usr/cygnus/progressive-97r2/share/bison.simple"

  yyvsp -= yylen;
  yyssp -= yylen;
#ifdef YYLSP_NEEDED
  yylsp -= yylen;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

  *++yyvsp = yyval;

#ifdef YYLSP_NEEDED
  yylsp++;
  if (yylen == 0)
    {
      yylsp->first_line = yylloc.first_line;
      yylsp->first_column = yylloc.first_column;
      yylsp->last_line = (yylsp-1)->last_line;
      yylsp->last_column = (yylsp-1)->last_column;
      yylsp->text = 0;
    }
  else
    {
      yylsp->last_line = (yylsp+yylen-1)->last_line;
      yylsp->last_column = (yylsp+yylen-1)->last_column;
    }
#endif

  /* Now "shift" the result of the reduction.
     Determine what state that goes to,
     based on the state we popped back to
     and the rule number reduced by.  */

  yyn = yyr1[yyn];

  yystate = yypgoto[yyn - YYNTBASE] + *yyssp;
  if (yystate >= 0 && yystate <= YYLAST && yycheck[yystate] == *yyssp)
    yystate = yytable[yystate];
  else
    yystate = yydefgoto[yyn - YYNTBASE];

  goto yynewstate;

yyerrlab:   /* here on detecting error */

  if (! yyerrstatus)
    /* If not already recovering from an error, report this error.  */
    {
      ++yynerrs;

#ifdef YYERROR_VERBOSE
      yyn = yypact[yystate];

      if (yyn > YYFLAG && yyn < YYLAST)
	{
	  int size = 0;
	  char *msg;
	  int x, count;

	  count = 0;
	  /* Start X at -yyn if nec to avoid negative indexes in yycheck.  */
	  for (x = (yyn < 0 ? -yyn : 0);
	       x < (sizeof(yytname) / sizeof(char *)); x++)
	    if (yycheck[x + yyn] == x)
	      size += strlen(yytname[x]) + 15, count++;
	  msg = (char *) malloc(size + 15);
	  if (msg != 0)
	    {
	      strcpy(msg, "parse error");

	      if (count < 5)
		{
		  count = 0;
		  for (x = (yyn < 0 ? -yyn : 0);
		       x < (sizeof(yytname) / sizeof(char *)); x++)
		    if (yycheck[x + yyn] == x)
		      {
			strcat(msg, count == 0 ? ", expecting `" : " or `");
			strcat(msg, yytname[x]);
			strcat(msg, "'");
			count++;
		      }
		}
	      yyerror(msg);
	      free(msg);
	    }
	  else
	    yyerror ("parse error; also virtual memory exceeded");
	}
      else
#endif /* YYERROR_VERBOSE */
	yyerror("parse error");
    }

  goto yyerrlab1;
yyerrlab1:   /* here on error raised explicitly by an action */

  if (yyerrstatus == 3)
    {
      /* if just tried and failed to reuse lookahead token after an error, discard it.  */

      /* return failure if at end of input */
      if (yychar == YYEOF)
	YYABORT;

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Discarding token %d (%s).\n", yychar, yytname[yychar1]);
#endif

      yychar = YYEMPTY;
    }

  /* Else will try to reuse lookahead token
     after shifting the error token.  */

  yyerrstatus = 3;		/* Each real token shifted decrements this */

  goto yyerrhandle;

yyerrdefault:  /* current state does not do anything special for the error token. */

#if 0
  /* This is wrong; only states that explicitly want error tokens
     should shift them.  */
  yyn = yydefact[yystate];  /* If its default is to accept any token, ok.  Otherwise pop it.*/
  if (yyn) goto yydefault;
#endif

yyerrpop:   /* pop the current state because it cannot handle the error token */

  if (yyssp == yyss) YYABORT;
  yyvsp--;
  yystate = *--yyssp;
#ifdef YYLSP_NEEDED
  yylsp--;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "Error: state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

yyerrhandle:

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yyerrdefault;

  yyn += YYTERROR;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != YYTERROR)
    goto yyerrdefault;

  yyn = yytable[yyn];
  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrpop;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrpop;

  if (yyn == YYFINAL)
    YYACCEPT;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting error token, ");
#endif

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  yystate = yyn;
  goto yynewstate;
}
#line 1580 "/5g/ian/binutils/release/copy/binutils/rcparse.y"


/* Set the language from the command line.  */

void
rcparse_set_language (lang)
     int lang;
{
  language = lang;
}
