typedef union {
  char *id;
  int number;
} YYSTYPE;
#define	NAME	258
#define	LIBRARY	259
#define	DESCRIPTION	260
#define	STACKSIZE	261
#define	HEAPSIZE	262
#define	CODE	263
#define	DATA	264
#define	SECTIONS	265
#define	EXPORTS	266
#define	IMPORTS	267
#define	VERSIONK	268
#define	BASE	269
#define	CONSTANT	270
#define	READ	271
#define	WRITE	272
#define	EXECUTE	273
#define	SHARED	274
#define	NONAME	275
#define	ID	276
#define	NUMBER	277


extern YYSTYPE yylval;
