
foo:	.ASSIGNA	1
	\&foo+1	
	\&foo+1
foo:	.ASSIGNC	"foo"
	\&foo+1
	\&foo+1

foo:	.ASSIGNA	1
	\&foo+1	
	\&foo+1
foo:	.ASSIGNC	"foo"
	\&foo+1
	\&foo+1

foo:	.ASSIGNA	1
	\&foo+1	
	\&foo+1
foo:	.ASSIGNC	"foo"
	\&foo+1
	\&foo+1

foo:	.ASSIGNA	1
	\&foo+1	
	\&foo+1
foo:	.ASSIGNC	"foo"
	\&foo+1
	\&foo+1

foo:	.ASSIGNA	1
	\&foo+1	
	\&foo+1
foo:	.ASSIGNC	"foo"
	\&foo+1
	\&foo+1
	foo
	foo	foo
	foo	foo
	foo
	.END

