# Test asi's.
	.text
	lduwa [%g1]0,%g2
	lduwa [%g1]255,%g2
	lduwa [%g1]#ASI_AIUP,%g2
	lduwa [%g1]#ASI_AIUS,%g2
	lduwa [%g1]#ASI_AIUP_L,%g2
	lduwa [%g1]#ASI_AIUS_L,%g2
	lduwa [%g1]#ASI_P,%g2
	lduwa [%g1]#ASI_S,%g2
	lduwa [%g1]#ASI_PNF,%g2
	lduwa [%g1]#ASI_SNF,%g2
	lduwa [%g1]#ASI_P_L,%g2
	lduwa [%g1]#ASI_S_L,%g2
	lduwa [%g1]#ASI_PNF_L,%g2
	lduwa [%g1]#ASI_SNF_L,%g2
	lduwa [%g1]#ASI_AS_IF_USER_PRIMARY,%g2
	lduwa [%g1]#ASI_AS_IF_USER_SECONDARY,%g2
	lduwa [%g1]#ASI_AS_IF_USER_PRIMARY_LITTLE,%g2
	lduwa [%g1]#ASI_AS_IF_USER_SECONDARY_LITTLE,%g2
	lduwa [%g1]#ASI_PRIMARY,%g2
	lduwa [%g1]#ASI_SECONDARY,%g2
	lduwa [%g1]#ASI_PRIMARY_NOFAULT,%g2
	lduwa [%g1]#ASI_SECONDARY_NOFAULT,%g2
	lduwa [%g1]#ASI_PRIMARY_LITTLE,%g2
	lduwa [%g1]#ASI_SECONDARY_LITTLE,%g2
	lduwa [%g1]#ASI_PRIMARY_NOFAULT_LITTLE,%g2
	lduwa [%g1]#ASI_SECONDARY_NOFAULT_LITTLE,%g2
