
/* Name of package.  */
#undef PACKAGE

/* Version of package.  */
#undef VERSION

/* Whether strstr must be declared even if <string.h> is included.  */
#undef NEED_DECLARATION_STRSTR

/* Whether free must be declared even if <stdlib.h> is included.  */
#undef NEED_DECLARATION_FREE

/* Whether sbrk must be declared even if <unistd.h> is included.  */
#undef NEED_DECLARATION_SBRK

/* Whether getenv must be declared even if <stdlib.h> is included.  */
#undef NEED_DECLARATION_GETENV
@TOP@

/* Do we need to use the b modifier when opening binary files?  */
#undef USE_BINARY_FOPEN
