# @OC@ test
	
text_label:

# Condition tests
	@OC@	text_label
	@OC@al	text_label
	@OC@ra	text_label
	@OC@eq	text_label
	@OC@z	text_label
	@OC@ne	text_label
	@OC@nz	text_label
	@OC@pl	text_label
	@OC@p	text_label
	@OC@mi	text_label
	@OC@n	text_label
	@OC@cs	text_label
	@OC@c	text_label
	@OC@lo	text_label
	@OC@cc	text_label
	@OC@nc	text_label
	@OC@hs	text_label
	@OC@vs	text_label
	@OC@v	text_label
	@OC@vc	text_label
	@OC@nv	text_label
	@OC@gt	text_label
	@OC@ge	text_label
	@OC@lt	text_label
	@OC@le	text_label
	@OC@hi	text_label
	@OC@ls	text_label
	@OC@pnz	text_label

	@OC@	external_text_label

	@OC@	0

# Delay slots
	@OC@.d	text_label
	@OC@.nd	text_label
	@OC@.jd	text_label

# Condition tests and delay slots
	@OC@eq.d	text_label
	@OC@ne.nd	text_label
	@OC@cc.jd	text_label
