typedef union
{
  char *string;
  struct string_list *list;
} YYSTYPE;
#define	CHECK	258
#define	CODESTART	259
#define	COPYRIGHT	260
#define	CUSTOM	261
#define	DATE	262
#define	DEBUG	263
#define	DESCRIPTION	264
#define	EXIT	265
#define	EXPORT	266
#define	FLAG_ON	267
#define	FLAG_OFF	268
#define	FULLMAP	269
#define	HELP	270
#define	IMPORT	271
#define	INPUT	272
#define	MAP	273
#define	MESSAGES	274
#define	MODULE	275
#define	MULTIPLE	276
#define	OS_DOMAIN	277
#define	OUTPUT	278
#define	PSEUDOPREEMPTION	279
#define	REENTRANT	280
#define	SCREENNAME	281
#define	SHARELIB	282
#define	STACK	283
#define	START	284
#define	SYNCHRONIZE	285
#define	THREADNAME	286
#define	TYPE	287
#define	VERBOSE	288
#define	VERSIONK	289
#define	XDCDATA	290
#define	STRING	291
#define	QUOTED_STRING	292


extern YYSTYPE yylval;
