SCRIPT_NAME=m68kaux
OUTPUT_FORMAT="coff-m68k-aux"
SEGMENT_SIZE=0x40000
TARGET_PAGE_SIZE=0x1000
TEXT_START_ADDR="$SEGMENT_SIZE + SIZEOF_HEADERS"
NON_PAGED_TEXT_START_ADDR=SIZEOF_HEADERS
DATA_ALIGNMENT_="(. & (-$SEGMENT_SIZE | $TARGET_PAGE_SIZE-1)) + $SEGMENT_SIZE"
ARCH=m68k
